/*****************************************************************************
 *  File: reconfig_utils.h
 *
 *  This file contains the constants, structure definitions, and funtion
 *  prototypes for the application selector.
 *
 ****************************************************************************/

#ifndef APP_SELECTOR_H_
#define APP_SELECTOR_H_

#include "sys/alt_flash_dev.h"

//#define DEBUG_MODE 1 // debugging help
//#define UPDATE_PFL 1 // option to keep or update PFL OPTION BITS

#define NUM_VIDEO_FRAMES 2

#define FLASH_SECTOR_SIZE            0x40000  //128KB
#define FLASH_BOOT_BLOCK_SECTOR_SIZE 0x10000  //32KB
#define HW_IMAGE_SIZE                2686976	// SIVGX530 0x148A4CB (pfl erases)
#define SECTORS_FOR_HW_IMAGE         ((HW_IMAGE_SIZE / FLASH_SECTOR_SIZE) + 1) // 5C
#define BYTES_FOR_HW_IMAGE           (SECTORS_FOR_HW_IMAGE * FLASH_SECTOR_SIZE)
#define BOARD_ID 					 0x1C     // used for board ID in flash

/****************************************************************
// SIVGX530
 Section                 Size  Address
 -----------------------------------------------------
 Top boot blocks        128KB  0x03FE0000 - 0x03FFFFFF
 -----------------------------------------------------
 User Software           24MB  0x034A0000 - 0x03FDFFFF
 User Hardware           12MB  0x02000000 - 0x0349FFFF
 ------------------------
 zipfs (html content)     8MB  0x01A60000 - 0x01FFFFFF
 Factory Software         8MB  0x014C0000 - 0x01A5FFFF
 Factory Hardware        12MB  0x00020000 - 0x014BFFFF
 -----------------------------------------------------
 PFL Option Bits         32KB  0x00018000 - 0x0001FFFF
 Board Settings          32KB  0x00010000 - 0x00017FFF
 Ethernet Option Bits    32KB  0x00008000 - 0x0000FFFF
 User Reset Vector       32KB  0x00000000 - 0x00007FFF

****************************************************************/

#define FACTORY_HW_IMAGE_PAGE        0
#define FACTORY_HW_IMAGE_OFFSET      0x00040000
#define FACTORY_SW_IMAGE_OFFSET      0x069C0000

// correct user #defines
#define USER_HW_IMAGE_PAGE           1
#define USER_HW_IMAGE_OFFSET         0x02940000
#define USER_SW_IMAGE_OFFSET         0x071C0000

#define RO_ZIPFS_OFFSET              0x061C0000
#define ETHERNET_OPTION_BITS         0x00010000
#define PFL_OPTION_BIT_OFFSET        0x00030000

#define SREC_BUFFER_SIZE             0x00020000

// These are the SIVGX SI 16bit masks for the MAXII reconfiguation 16bit registers:
#define MAXII_CLEAR_PSR_MASK         0xFFFE // 1111 1111 1111 1110 | 0
#define MAXII_CLEAR_PSO_MASK         0xFFFD // 1111 1111 1111 1101 | 1
#define MAXII_FORCE_RECONFIG_MASK    0x000B // 0000 0000 0000 1011 | B
#define MAXII_CLEAR_PSS_MASK         0xFFF7 // 1111 1111 1111 0111 | 3

//#define MAXII_RSU_REG				0x6 // was 0x8
#define MAXII_RSU_REG  0x8 			// MAXII_REG2 is 0x8

#define PAGE_USER				1	// NIOS_PIO_PAGE_SELECT_BASE
#define SRST					0	// NIOS_PIO_RECONFIGURE_N_BASE

#define USE_HAL_FLASH_DRIVER            0
#define USE_INTEL_P30_FLASH_DRIVER      1

// Error Codes
#define SUCCESS                      0
#define FAILURE                     -1
#define ERROR_FAT_FAILED_TO_MOUNT   -2
#define ERROR_FAT_COULD_NOT_FIND    -3
#define ERROR_FAT_READ_ZERO_BYTES   -4
#define ERROR_SREC_HAS_NO_TIMESTAMP -5
#define ERROR_COULD_NOT_ALLOCATE    -6

// Filetypes
#define FLASH                        1
#define RBF                          2

#define FILE_BUFFER_SIZE FLASH_SECTOR_SIZE

int ReconfigFPGA( unsigned int hw_flash_page );
void InitializePFLOptionBits( unsigned int option_bit_offset );

#endif /*APP_SELECTOR_H_*/
